
### Prerequisites

Three main libraries are required to run the code : Dscribe (SOAP descriptors), Scikit-learn (Machine Learning tools) and ASE (Atomic Simulation Environment). 

More precisely :   
- python==3.10
- ase==3.22.1  
- scikit-learn==1.4.1
- dscribe==2.1.0
- numpy==1.26.4
- scipy==1.12.0
- matplotlib==3.8.3 

All libraries can be installed via `pip` , e.g.
```
pip install -r requirements.txt
```


## Usage

The code builts Energy Adsorption Maps (EAMs) using machine learning.

The file "run.py" (i) trains a model from a training set containing adsorption energies calculated on selected sites (".traj" file), and (ii) predicts adsorption energies of new sites located on a $n$ $\times$ $n$ regular grid. Results are written in the file "result.txt".

The ".traj" file contains data useful to describe the simulation box, the atomic positions, the grid ($n$) as well as parameters for the SOAP descriptors and the Gaussian Process Regression. 

The code to create the Adsorption Enery Map ("Ag111_AEM.pdf", Working Example) is also given and uses the matplotlib package. Code execution must take place in the same directory as the file "result.txt" previously obtained.

## Data sets

Two data bases are provided. 

The minimum working exemple deals with atomic Hydrogen adsorption on Ag(111), and the training set contains DFT data calculated on 3 distinct surface sites. 

The data set corresponding to atomic Hydrogen adsorption on Al<sub>13</sub>Co<sub>4</sub>(100) is the one used in the [paper](https://doi.org/10.1021/acs.jctc.4c00367). The file "final.traj" located in the folder "data_Al13Co4" contains DFT relaxations calculated in 20 $\times$ 20 surface sites. To obtain the results published in the [paper](https://doi.org/10.1021/acs.jctc.4c00367), a "train.traj" file has to be generated from a fraction of the "final.traj" file. The choice of relevant sites can be achieved with the code "position_selection.py" which uses the Farthest Point Sampling method ; The number of selected positions is given by the integer "ncomp" (first line of the code). The training can be performed with the "run.py" code. Both files ("train.traj" and "run.py") have to be located in the same directory. 


 ## Citation
```
@article{Boulangeot24hyd,
author    = {Nathan Boulangeot and Florian Brix and Frédéric Sur and Emilie Gaudry},
title     = {Hydrogen, oxygen and lead adsorbates on Al$_{13}$Co$_4$(100) : accurate potential energy surfaces at low computational cost by machine learning and DFT-based data},
journal   = {Journal of Chemical Theory and Computation},
year      = {2024},
doi       = {10.1021/acs.jctc.4c00367}
} 
```





