import os
import numpy as np
from sklearn.metrics import pairwise_kernels
from ase.io import Trajectory
from scipy.spatial import distance_matrix
from sklearn.preprocessing import StandardScaler
from codes.tool import *

def retrieve_pos_ind(traj,atom=0):
      postrain=None
      ind_split=[]
      for ind,atoms in enumerate(traj):
          pos=atoms.get_positions()[atom][:2]
          if postrain is None:
             postrain=[pos]
             count=0
          elif  any(np.equal(postrain,pos).all(1)):
                pass
          else:
                postrain=np.vstack((postrain,pos))
                ind_split.append([count,ind-1])
                count=ind
      ind_split.append([count,len(traj)-1])
      return np.array(ind_split)

def matrice_farthest_search(points, k,mode='euclidian',gamma=1e-5,n=3):
         if mode == 'euclidian':
             def distance_m(a):
                return distance_matrix(a,a)
         if mode == 'rbf':
              def distance_m(a):
                DM=pairwise_kernels(a,a,metric='rbf',gamma=gamma)
                return 1-DM
         DM=distance_m(points)
         remaining_points = list(points)
         solution_set = []
         indices=[]
         indices_cluster=[]
         indiceleft=list(range(len(remaining_points)))
         #init=indiceleft.pop(random.randint(0, len(indiceleft) - 1))
         init=indiceleft.pop(np.argmin(np.mean(DM,axis=0)))         
         indices.append(init)
         for _ in range(k-1):
             a=[min([DM[p][pl] for p in indices])for pl in indiceleft]
             ind_rem=a.index(max(a))             
             rem=indiceleft.pop(ind_rem)
             indices.append(rem)   
             solution_set.append(max(a))
         return np.array(indices)

def train_from_pos(ncomp,data_folder='data_Al13Co4',traj='final.traj', atomseul='H'):
    os.chdir(data_folder)
    sys=Trajectory(traj)  
    species=[]
    elem=sys[0].get_chemical_symbols() 
    #adsorbate index
    for el in range(len(elem)): 
        if elem[el] not in species:
            species.append(elem[el])
            if elem[el]== atomseul:
                ats=el

    # full data set
    params={'species':species,'l_max':3,'n_max':6,'r_cut':6}
    desc=create_descriptor(method='soap',params=params,ats=ats)
    X_ini=desc.create(sys,load=False)
     
    # positions are compared in their initial configurations (POSCAR) 
    # !! to discriminate positions in the "final.traj" data set, the function
    # retrieve_pos_ind look for diffences in adsorbate (x, y) values.
    index_pos_100=retrieve_pos_ind(sys)
    X_poscar=[X_ini[i[0]] for i in index_pos_100]
         
    #scaling
    scaler=StandardScaler()
    scaler.fit(X_poscar)
    X_poscar_T=scaler.transform(X_poscar)
    indices=matrice_farthest_search(X_poscar_T, ncomp,)
    print(indices)
    train_set=Trajectory('train.traj','w')
    for ind in indices:
        #write the whole DFT relaxation for each position selected
        for i in range(index_pos_100[ind,0],index_pos_100[ind,1]+1):
            train_set.write(sys[i])
    train_set.close()

if __name__ == '__main__':
        ncomp=64
        train_from_pos(ncomp,data_folder='data_Al13Co4',traj='final.traj', atomseul='H')